//=======================================================================
// bits.cc : inspecter le codage de types primitifs dans la mmoire
// Copyright (C) 2006-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//=======================================================================

#include <iostream>
using namespace std;

// Le type de base qui nous intresse ici est l'octet (= 8 bits)
typedef unsigned char Byte;

// Afficher les 8 bits d'un octet dans le sens du poids dcroissant
void affiche_bits( Byte b )
{
  for( int bit=7; bit>=0; --bit ) 
    cout << ( ( b >> bit ) & 1 );
  cout << " ";
}

// Afficher une variable et son codage dans la mmoire (dcroissant)
template <typename T>
void inspecte( const T& objet )
{
  cout << "valeur de la variable : " << objet << endl;
  cout << "codage en mmoire     : ";
  const Byte* pointer= static_cast<const Byte*>(static_cast<const void*>( &objet ));
  for( int i= sizeof(T)-1; i>=0; --i )
    affiche_bits( pointer[i] );
  cout << endl;
}

int main()
{
  cout << "\nBonjour et bienvenue aux inspections des bits !\n" 
       << "Les exemples suivants montrent comment sont stocks\n"
       << "les types primitif du C++ dans la mmoire vive.\n"
       << "(Convention \"little-endian\" & affichage inverse)\n"
       << endl;

  cout << "char (8 bits)         : 'A', code ASCII : 65" << endl;
  inspecte( char('A') );
  cout << endl;
  
  cout << "short int (16 bits)   : 11 + 256*5" << endl;
  inspecte( int(11 + 256*5) );
  cout << endl;

  cout << "int (32 bits)         : a = 11 + 256 * ( 5 + 256 * ( 15 + 256*31 ))" << endl;
  inspecte( int( 11 + 256 * ( 5 + 256 * ( 15 + 256*31 )) ) );
  cout << endl;

  cout << "double (64 bits)      : signe (1 bit), exposant+1023 (11bits), mantisse rduite (52 bits)" << endl;
  for( double s=2.0, a=1.0; a>=0.03125; s+=a, a/=2 ) inspecte(s);
  for( double a=1.0; a>=0.0625; a/=2 ) inspecte(a);
  cout << endl;
}
