//=======================================================================
// chrono.cc : exemple de chronomtrage d'une boucle
// Copyright (C) 2004-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Description :
// Ce programme msure le temps d'excution d'une boucle.
// Les deux fonctions suivantes calculent la somme 1+2+...+n, 
// la premire par la formule n(n+1)/2, la deuxime par une boucle.
//
// Compilation :
// On peut tester la performance de notre compilateur GNU g++
// en matire d'optimisation. Pour ceci compiler une fois avec g++
// (sans optimisation), puis avec g++ -O1, g++ -O2, et g++ -O3 
// (optimisation de plus en plus sophistique). Sachez toutefois
// que le meilleur compilateur ne remplacera jamais le bon choix
// d'algorithme, ici le choix d'une formule beaucoup plus efficace !
//=======================================================================

#include <iostream>      // dclarer l'entre-sortie standard
#include "timer.hh"      // horloge pour chronomtrer l'excution
using namespace std;     // accs direct aux fonctions standard

typedef long long int Integer;

// Calcul de la somme 1+2+...+n par la formule n(n+1)/2
Integer fonction1( Integer n )
{
  if ( n%2 == 0 )
    return (n/2)*(n+1);
  else
    return n*((n+1)/2);
}

// Calcul de la somme 1+2+...+n par une boucle
Integer fonction2( Integer n )
{
  Integer somme= 0;
  for( Integer k=1; k<=n; ++k )
    somme+= k;
  return somme;
}

// Chronomtrage des deux fonctions
int main()
{
  cout << "\nBonjour et bienvenue ! Ce logiciel compare le temps de calcul\n"
       << "de la somme 1+2+...+n par la formule n(n+1)/2 et par une boucle.\n";
  cout.setf( ios::fixed );
  cout.precision(2);
  Timer timer;

  for ( Integer n= 1000000; n <= 100000000000ll; n*=10 )
    {
      cout << "\nOn choisit n=" << n << endl;
      
      cout << "Calcul par la formule... " << flush;
      timer.start();
      Integer somme1= fonction1(n);
      timer.stop();
      cout << "somme = " << somme1 
	   << "   temps coul : " << timer << endl;
      
      cout << "Calcul par une boucle... " << flush;
      timer.start();
      Integer somme2 = fonction2(n);
      timer.stop();
      cout << "somme= " << somme2 
	   << "   temps coul : " << timer << endl;
    }

  cout << "\nAu revoir.\n" << endl;
}
