//=======================================================================
// karatsuba-test.cc : multiplication rapide selon Karatsuba
// Copyright (C) 2004-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
//  compiler avec l'option -O3 pour optimiser la rapidit. 
//
// Test empirique :
// Ce programme compare la multiplication selon Karatsuba avec la mthode 
// scolaire (de complexit quadratique). On constate deux phnomnes :
// - chaque fois que la longueur double, le temps d'excution est
//   multipli par 4 pour la mthode scolaire, mais seulement par 3 
//   pour Karatsuba. (Celui-ci est donc de complexit O(m^1.585).)
// - Karatsuba est plus rapide que la mthode scolaire pour des grands
//   nombres. Le programme permet de dterminer la taille critique 
//    partir de laquelle Karatsuba s'amortit.
//=======================================================================

#include "karatsuba.cc"        // notre implmentation faite maison
#include "timer.hh"            // horloge pour chronomtrer l'excution
using namespace std;           // accs direct aux fonctions standard

int main()
{
  cout << "\nChronomtrage de la multiplication de deux entiers." << endl;
  Naturel x= 10, y, z;
  Timer timer;
  for( int i=0; i<20; ++i )
    {
      cout.width(10);
      cout << x.size() << " dcimales : " << flush;
      z= x;

      cout << "karatsuba " << flush;
      timer.start();
      karatsuba( z, z+1, x );
      timer.stop();
      cout << timer << flush;

      cout << ", scolaire " << flush;
      timer.start();
      multiplication( z, z+1, y );
      timer.stop();
      cout << timer << flush;

      if ( x == y ) 
	cout << ", ok." << endl; 
      else
	cout << ", ERREUR !" << endl;
    };  
  cout << "Au revoir.\n" << endl;
}
