//=======================================================================
// racine.cc : tester deux mthodes pour calculer la racine entire
// Copyright (C) 2004-2006 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
// 
// Description :
// Ce programme compare deux mthodes pour calculer la racine entire :
// la recherche dichotomique et la mthode de Newton-Hron.
// 
// Compilation :
//  compiler avec l'option -lgmpxx pour inclure la bibliothque GMP,
// puis avec l'option -O3 pour effectuer quelques petites optimisations.
//=======================================================================

#include <iostream>
#include <gmpxx.h>
#include <vector>
#include "timer.hh"
using namespace std;

typedef mpz_class Integer;

// Dterminer le logarithme en base 2 (version optimise pour GMP)
int log2( const Integer& y )
{ 
  return mpz_sizeinbase( y.get_mpz_t(), 2 ) - 1; 
}

// Calculer la puissance x^n (exercice)
Integer puissance( Integer x, int n );

// Calculer la racine n-ime entire par une recherche dichotomique (exercice)
Integer racine_dichotomique( const Integer &y, int n );

// Calculer la racine n-ime entire par la mthode de Newton-Hron (exercice)
Integer racine_newton( const Integer &y, int n );

// Comparer les deux mthodes 
int main()
{
  Timer timer;
  const int max= 500;
  vector<Integer> resultat(max+1);
  cout << "Calcul de (n!)^(1/3) pour n=2,...," << max << endl;

  cout << "Par la mthode dichotomique : " << endl;
  Integer f= 1;
  timer.start();
  for ( int n=2; n<=max; ++n )
    {
      f*= n;
      resultat[n]= racine_dichotomique(f,3);
      cout << "." << flush;
    };
  timer.stop();
  cout << "\nTemps coul : "<< timer << endl;

  cout << "Par la mthode de Newton-Hron : " << endl; 
  f= 1;
  timer.start();
  for ( int n=2; n<=max; ++n )
    {
      f*= n;
      if ( resultat[n] != racine_newton(f,3) )
	cout << "!" << flush; 
      else
	cout << "." << flush;
    };
  timer.stop();
  cout << "\nTemps coul : "<< timer << endl;
  cout << "Au revoir.\n" << endl;
}
