//=======================================================================
// vectorio.cc : entre-sortie gnrique pour les vecteurs
// Copyright (C) 2004-2007 by Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Description :
// Ce fichier implmente un oprateur de sortie et un oprateur d'entre 
// pour les vecteurs. Le format utilis pour l'entre-sortie d'un vecteur
// v de taille n est ( v[0] v[1] ... v[n-1] ) .  noter que l'on a choisi 
// ici de sparer les lments par des espaces (et non par des virgules).
//
// Implmentation :
// Au lieu d'crire une fonction pour chaque type possible, on opte ici
// pour une implmentation gnrique. Ainsi les deux oprateurs suivants
// sont des modles qui dpendent d'un paramtre T : c'est le type des 
// lments, encore indtermin. Vous pouvez utiliser ces oprateurs
// de manire usuelle, comme indique dans la fonction main() plus bas.
// Le compilateur traduira le modle en une fonction concrte, appele 
// instance du modle, pour chaque type en question.
//=======================================================================

#include <iostream>           // dclarer l'entre-sortie standard
#include <vector>             // dfinir la classe gnrique vector
#include <string>             // dclarer la classe string 
using namespace std;          // accs direct aux fonctions standard

//-----------------------------------------------------------------------
// Oprateur de sortie (comme d'habitude assez vident)
//-----------------------------------------------------------------------
template <typename T>
ostream& operator << ( ostream& out, const vector<T>& vec )
{
  out << "( ";
  for ( size_t i=0; i<vec.size(); ++i )
    out << vec[i] << " ";
  out << ")";
  return out;
}

//-----------------------------------------------------------------------
// Oprateur d'entre (comme d'habitude un peu plus subtil)
//-----------------------------------------------------------------------
template <typename T>
istream& operator >> ( istream& in, vector<T>& vec )
{
  // remettre le flot  l'tat normal (si possible)
  in.clear();
  if ( in.bad() ) return in;
  
  // lire la parenthse ouvrante (erreur sinon)
  char symbole;
  in >> symbole;
  if ( in.fail() ) return in;
  if ( symbole != '(' )
    {
      in.putback( symbole );
      in.setstate( ios::failbit ); 
      return in;
    }
  
  // lire les lments un par un, jusqu' la parenthse fermante
  vec.clear();
  T element;
  for( in >> symbole; symbole!=')'; in >> symbole )
    {
      if ( in.fail() ) return in;
      in.putback( symbole );
      in >> element;
      if ( in.fail() ) return in;
      vec.push_back( element );
    }
  return in;
}

//-----------------------------------------------------------------------
// Dterminer l'tat d'un flot d'entre
//-----------------------------------------------------------------------
string status( const istream& in )
{
  if ( in.bad() )  return string("status : bad");
  if ( in.fail() ) return string("status : fail");
  if ( in.eof() )  return string("status : eof");
  if ( in.good() ) return string("status : good");
  return string("status : unknown");
}

//-----------------------------------------------------------------------
// Un petit programme pour illustrer et tester l'entre-sortie. ( mettre 
// en commentaire aprs des tests : vous pouvez ainsi inclure ce fichier
// avec la directive #include "vectorio.cc" dans vos propres programmes.)
//-----------------------------------------------------------------------
/*
int main()
{
  cout << "\nBonjour et bienvenue aux vecteurs !" << endl;

  vector<int> a;
  cout << endl << "Entrez un vecteur <int> svp : ";
  cin >> a;
  cout << status(cin) << endl << "Vous avez entr " << a << endl;

  vector<double> b;
  cout << endl << "Entrez un vecteur <double> svp : ";
  cin >> b;
  cout << status(cin) << endl << "Vous avez entr " << b << endl;

  vector<char> c;
  cout << endl << "Entrez un vecteur <char> svp : ";
  cin >> c;
  cout << status(cin) << endl << "Vous avez entr " << c << endl;

  vector< vector<int> > d;
  cout << endl << "Entrez un vecteur de vecteurs <int> svp : ";
  cin >> d;
  cout << status(cin) << endl << "Vous avez entr " << d << endl;

  cout << endl << "Au revoir." << endl;
}
*/
