//=======================================================================
// rsa.cc : implmentation basique du cryptosystme RSA
// Copyright (C) 2006-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
// 
//  compiler avec l'option -lgmpxx pour inclure la bibliothque GMP.
// La classe mpz_class (appele Integer dans la suite) implmente
// l'arithmtique des grands entiers, sans limitation de taille.
// 
// Les fonctions manquantes sont  implmenter en exercice
// (voir le cours et les commentaires plus bas).
//=======================================================================

#include <iostream>        // entre-sortie standard
#include <vector>          // classe gnrique vector 
#include <string>          // classe string 
using namespace std;       // accs direct aux fonctions standards

#include <gmpxx.h>         // la bibliothque gmp pour C++
typedef mpz_class Integer; // le synonyme `Integer' semble plus parlant

//-----------------------------------------------------------------------
// Construire un nombre entier pseudo-alatoire
// Entre : deux entiers min <= max
// Sortie : un entier dans l'intervalle [min,max]
//-----------------------------------------------------------------------
Integer random( const Integer& min, const Integer& max )
{
  static gmp_randclass generateur(gmp_randinit_default);
  return generateur.get_z_range(max-min+1) + min;
}

//-------------------------------------------------------------------------
// Dterminer le reprsentant prfr de a modulo m : si m est non nul, 
// on renvoie l'unique entier r avec  0 <= r < abs(m) qui est congru  a
// modulo m.  De manire analogue, si m est nul, on renvoie a simplement.
//-------------------------------------------------------------------------
Integer pmod( const Integer& a, const Integer& m )
{
  if ( m == 0 ) return a;
  Integer r= a%m;
  if ( r<0 ) r+= abs(m);
  return r;
}

//-----------------------------------------------------------------------
// Calculer  base^exp modulo mod  par une puissance modulaire rapide.
// Entre : un entier base, un entier exp >= 0, un entier mod != 0
// Sortie : l'unique entier dans 0,...,mod-1 congru  base^exp modulo mod
//-----------------------------------------------------------------------
Integer puissance( Integer base, Integer exp, const Integer& mod )
{
  // 
  // Exercice (environ 15 lignes)
  //
}

//-------------------------------------------------------------------------
// Calculer l'inverse de a modulo mod, s'il existe, renvoyer zro sinon.
// Pour rappel : l'inverse de a modulo mod est l'unique entier u dans 
// l'intervalle 1,...,mod-1 tel que a*u soit congru  1 modulo mod.
// (A noter que l'on prend systmatiquement le reprsentant prfr.)
//
// Entre : deux entiers a et mod
// Sortie : l'inverse de a modulo mod, si a est inversible, zro sinon
//-------------------------------------------------------------------------
Integer inverse( Integer a, const Integer& mod )
{
  // 
  // Exercice (environ 15 lignes)
  //
}

//-------------------------------------------------------------------------
// Renverser l'ordre des lments dans un vecteur 
//-------------------------------------------------------------------------
template <typename Vector>
void renverser( Vector& vec )
{
  for( size_t i=0, j=vec.size()-1; i<j; ++i, --j )
    swap( vec[i], vec[j] );
}

//-------------------------------------------------------------------------
// Calculer un entier positif  partir de chiffres dans une base donne.
// Entre : des chiffres (c_k,c_{k-1},...,c_1,c_0) en base b
// Sortie : l'entier c_k * b^k + c_{k-1} * b^{k-1} + ... + c_1 * b + c_0
//
// Attention :
// Ici les chiffres sont stocks en commenant par c_k avec le plus grand 
// poids b^k.  Ce n'est qu'une convention, mais une fois fixe elle doit 
// tre respecte partout, sinon les rsultats risquent d'tre incohrents.
//-------------------------------------------------------------------------
Integer convert( const vector<Integer>& chiffres, const Integer& base ){
  // 
  // Exercice (environ 7 lignes)
  //
}

//-------------------------------------------------------------------------
// Dcomposer un entier positif en chiffres dans une base donne.
// Entre : un entier n et une base b
// Sortie : les chiffres (c_k,c_{k-1},...,c_1,c_0) en base b 
//   tels que n = c_k * b^k + c_{k-1} * b^{k-1} + ... + c_1 * b + c_0
//
// Attention :
// Il est important d'utiliser la mme convention qu'avant concernant
// l'ordre des chiffres ! 
//-------------------------------------------------------------------------
vector<Integer> convert( Integer nombre, const Integer& base )
{
  // 
  // Exercice (environ 8 lignes)
  //
}

//-------------------------------------------------------------------------
// Remplacer les lettres minuscules a,...,z par les majuscules A,...,Z,
// ainsi que tout autre caractre par un espace '_' (underscore).
//-------------------------------------------------------------------------
void majuscules( string& texte )
{
  for( size_t i=0; i<texte.size(); ++i )
    {
      // Une lettre majuscule reste inchange
      if ( texte[i] >= 'A' && texte[i] <= 'Z' ) 
	continue;

      // Une lettre minuscule est remplace
      if ( texte[i] >= 'a' && texte[i] <= 'z' ) 
	{ 
	  texte[i]+= 'A' - 'a'; 
	  continue; 
	};

      // Tout autre caractre est annul
      texte[i]= '_';
    }
}

//-------------------------------------------------------------------------
// Transformer un texte en nombre entier.
// On travaille en base 27 et on interprte les lettres A,...,Z comme 
// chiffres 1,...,26, et tout autre caractre comme chiffre 0.
//-------------------------------------------------------------------------
Integer convert( const string& texte )
{
  // 
  // Exercice (environ 8 lignes)
  //
}

//-------------------------------------------------------------------------
// Transformer un nombre entier en texte.
// On travaille en base 27 et on reprsente les chiffres 1,...,26 par 
// les lettres majuscules A,...,Z, et le chiffre 0 par '_' (underscore).
//-------------------------------------------------------------------------
string convert( Integer nombre )
{
  // 
  // Exercice (environ 9 lignes)
  //
}

//-------------------------------------------------------------------------
// Crypter un texte selon la mthode RSA.
// Entre : un entier mod > 1, un exposant exp > 1, un texte
// Sortie : le texte crypt
//
// Ide de la procdure :
// - Le texte en clair t est transform en un entier n.
// - L'entier n est dcomposer en chiffres n_k,...,n_0 en base mod 
// - On applique RSA  chaque chiffre n_i pour obtenir n'_i
// -  partir des chiffres n'_k,...,n'_0 on dduit l'entier n' 
// -  partir de l'entier n' on dduit le texte crypt t'
//-------------------------------------------------------------------------
void cryptageRSA( const Integer& mod, const Integer& exp, string& texte )
{
  // 
  // Exercice (environ 10 lignes avec commentaires)
  //
}

//-------------------------------------------------------------------------
// Cryptage d'un texte selon RSA.
// Lecture des donnes du flot in, criture du rsultat dans le flot out.
//-------------------------------------------------------------------------
void cryptageRSA( istream& in, ostream& out )
{
  // Lire les paramtres : mod, exp, texte clair
  Integer mod, exp;
  string texte;
  in >> mod >> exp >> texte;

  // Appliquer le cryptage RSA comme implmente ci-dessus
  majuscules( texte );
  cryptageRSA( mod, exp, texte );

  // crire le rsultat : mod, exp, texte crypt
  out << mod << " " << exp << " " << texte << endl;
}  

//-----------------------------------------------------------------------
// Le test de pseudo-primalit selon Miller-Rabin
// Entre : un entier impair n>=5  tester et une base x
// Sortie : true si n est pseudo-premier  base x, false sinon
//
// NB: pour des raisons d'efficacit on passe aussi la dcomposition 
// n-1 = q 2^e avec q impair, qui est prcalcule une fois pour toute.
//-----------------------------------------------------------------------
bool estPseudoPremier( const Integer& n, const Integer& x, 
		       const Integer& q, const Integer& e )
{
  // 
  // Exercice (environ 10 lignes)
  //
}

//-----------------------------------------------------------------------
// Le test de primalit selon Miller-Rabin, randomis et itr
// Entre : un entier n
// Sortie : si 'false' alors on est sr que n est compos,
//          si 'true' alors n est probablement premier.
//-----------------------------------------------------------------------
bool estProbablementPremier( Integer n, int essais= 100 )
{
  // 
  // Exercice (environ 20 lignes avec commentaires)
  //
}

//-------------------------------------------------------------------------
// Produire un nombre premier alatoire entre pmin et pmax
// Entre : un intervalle [pmin,pmax] contenant au moins un nombre premier
// Sortie : un nombre pseudo-premier alatoire dans cet intervalle
//-------------------------------------------------------------------------
Integer produirePremierAleatoire( Integer pmin, Integer pmax )
{
  // 
  // Exercice (environ 7 lignes)
  //
}

//-------------------------------------------------------------------------
// Produire une clef alatoire pour le cryptage RSA 
// Entre : le nombre de bits souhait pour les nombres premiers p et q
// Sortie : une clef complte (n,c,d) pour le protocole RSA
//-------------------------------------------------------------------------
void produireClefRSA( int bits, Integer& n, Integer& c, Integer& d )
{
  // 
  // Exercice (environ 15 lignes avec commentaires)
  //
}

//-------------------------------------------------------------------------
// La fonction main gre la communication avec le monde
//-------------------------------------------------------------------------
int main()
{
  cryptageRSA( cin, cout );

  /*
  cout << "Bonjour et bienvenue au cryptage selon RSA." << endl;
  cout << "Entrez un entier et une base svp : ";
  Integer n,b;
  cin >> n >> b;
  vector<Integer> vec= convert( n , b );
  cout << "chiffres : ";
  for( size_t i = 0 ; i < vec.size(); i++ )
    cout << vec[i] << " ";
  cout << endl;
  Integer m= convert( vec, b );
  cout << "reconvertit : " << m << endl;
  
  cout << "Entrez un mot svp : ";
  string mot;
  cin >> mot;
  majuscules(mot);
  cout << "votre mot : " << mot << endl;
  Integer nombre= convert(mot);
  cout << "le nombre associ : " << nombre << endl;
  mot= convert(nombre);
  cout << "le mot reconvertit : " << mot << endl;
  */
}
