//=======================================================================
// euclide.hh : calcul du pgcd dans un anneau euclidien
// Copyright (C) 2003-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//=======================================================================

#ifndef __euclide_hh__
#define __euclide_hh__

// L'algorithme d'Euclide pour calculer le pgcd de a et b
template <typename Anneau>
Anneau pgcd( Anneau a, Anneau b )
{
  Anneau r;
  while( !zero(b) ) { r= eumod(a,b); a= b; b= r; };
  return assopref(a);
}

// L'algorithme d'Euclide tendu pour calculer le pgcd de a et b
// et des coefficients de Bzout u,v tels que pgcd(a,b) = au + bv
template <typename Anneau>
Anneau pgcd( Anneau a, Anneau b, Anneau& u, Anneau& v )
{
  u= Anneau(1); v= Anneau(0);
  Anneau x(0), y(1), q, r;
  while( !zero(b) )
    {
      eudiv(a,b,q,r); a= b; b= r;
      r= u - q * x;   u= x; x= r;
      r= v - q * y;   v= y; y= r;
    };
  a= assopref(a,q); u*= q; v*= q;
  return a;
}

// L'algorithme d'Euclide tendu pour calculer le pgcd de a et b
// et un coefficient de Bzout u tel que pgcd(a,b) = au modulo b
template <typename Anneau>
Anneau pgcd( Anneau a, Anneau b, Anneau& u )
{
  u= Anneau(1);
  Anneau x(0), q, r;
  while( !zero(b) )
    {
      eudiv(a,b,q,r); a= b; b= r;
      r= u - q * x;   u= x; x= r;
    };
  a= assopref(a,q); u*= q;
  return a;
}

#endif // __euclide_hh__
