//=======================================================================
// polynome.cc : classe pour modliser les polynmes sur un anneau
// Copyright (C) 2004-2006 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Ce petit programme illustre l'usage de la classe gnrique Polynome<A>
// qui modlise les polynmes sur un anneau A. Il permet aussi de tester
// si l'entre-sortie et les oprations arithmtiques fonctionnent bien.
//=======================================================================

#include <iostream>
#include <iomanip>
#include "rationnel.cc"
#include "pecoprem.hh"
#include "polynome.hh"
using namespace std;

template <typename Poly>
void test( Poly a, Poly b );

template <typename Poly>
void experimenter(void);

int main()
{
  cout << "Bienvenue aux polynmes !" << endl
       << "(1) coefficients entiers" << endl
       << "(2) coefficients rationnels" << endl
       << "(3) coefficients dans F11" << endl;
  //   << "(4) coefficients dans Q[T]" << endl;
  cout << "Quel est votre choix svp ? ";
  int choix;
  cin >> choix;
  switch( choix )
    {
    case 1:
      {
	cout << endl << "Polynmes  coefficients entiers. Exemple :" << endl;
	typedef Polynome<Integer> Poly;
	Poly p= Poly(1,2) + Poly(-4,3) - Poly(1,5);
	Poly q= Poly(2,0) + Poly(1,2);
	test(p,q);
	experimenter<Poly>();
	break;
      };
    case 2:
      {
	cout << endl << "Polynmes  coefficients rationnels. Exemple :" << endl;
	typedef Polynome<Rationnel> Poly;
	Poly::setVarSym('T');
	Poly p= Poly(1,2) + Poly(-4,3) - Poly(1,5);
	Poly q= Poly(2,0) + Poly(1,2);
	test(p,q);
	experimenter<Poly>();
	break;
      };    
    case 3:
      {
	cout << endl << "Polynmes  coefficients dans F11. Exemple :" << endl;
	typedef GF<11> F;
	typedef Polynome<F> Poly;
	Poly::setVarSym('T');
	Poly p= Poly(1,2) + Poly(-4,3) - Poly(1,5);
	Poly q= Poly(2,0) + Poly(1,2);
	test(p,q);
	experimenter<Poly>();
	break;
      };
    /*
    case 4:
      {
	cout << endl << "Polynmes  coefficients dans Q[T]. Exemple :" << endl;
	typedef Polynome<Rationnel> QT;
	QT::setVarSym('T');
	typedef Polynome<QT> Poly;
	Poly::setVarSym('X');
	Poly p("( TX - (2T-T3)X2 + (3+4T2)X3 + 5X4 )");
	Poly q("( 2T + 1X )");
	test(p,q);
	experimenter<Poly>();
	break;
      };
    */
    default: 
      cout << "L'option (" << choix << ") n'est pas disponible, dsol." << endl;
    }
  cout << "Au revoir." << endl;
}

void check( bool b )
{
  if ( !b )
    cout << "*** Aprs vrification ce rsultat me semble erron !" << endl;
}

template <typename Poly>
void test( Poly a, Poly b )
{
  cout << "a   = " << a << endl
       << "b   = " << b << endl;
  Poly c= a+b;
  cout << "a+b = " << c << endl;
  Poly d= a-b;
  cout << "a-b = " << d << endl;
  check( d+b == a );
  
  cout << "a*b = " << a*b << endl;
  /*
  cout << "a(b) = " << a(b) << endl
       << "b(a) = " << b(a) << endl;
  */
  
  Poly q,r;
  if ( divisible(a,b,q) )
    {
      cout << "a/b = " << q << " sans reste" << endl;
      check( a == b*q );
    }
  else
    {
      int k= pseudo_division(a,b,q,r);
      if ( k == 0 )
	{
	  cout << "division euclidienne  a = bq + r  avec" << endl
	       << "  q = " << q << endl
	       << "  r = " << r << endl;
	  check( a == b*q+r );
	}
      else if ( k > 0 )
	{
	  cout << "pseudo-division  c^k a = bq + r  avec k=" << k << endl
	       << "  q = " << q << endl
	       << "  r = " << r << endl;
	  typename Poly::Coeff f(1);
	  for( int i=1; i<=k; ++i ) f*= b.dom();
	  check( f*a == b*q+r );
	};
    };
}

template <typename Poly>
void experimenter( void )
{
  for(;;)
    {
      cout << endl << "Entrez deux polynmes svp : " << endl;
      Poly a,b;
      cin >> a >> b;
      if ( cin.bad() )  cout << "status : bad " << endl;
      if ( cin.fail() ) cout << "status : fail" << endl;
      if ( cin.eof() )  cout << "status : eof " << endl;
      if ( cin.good() ) cout << "status : good" << endl;

      test(a,b);
      return;

      cout << endl;
      for(;;)
	{
	  cout << "Voulez-vous continuer ? (o/n) ";
	  char reponse;
	  cin >> reponse;
	  if ( reponse == 'n' ) return;
	  if ( reponse == 'o' ) break;
	};
    };
}

