//=======================================================================
// racine.cc : calcul d'une racine n-ime
// Copyright (C) 2004-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Description :
// La fonction racine ci-dessous calcule une valeur approche de 
// la racine n-ime d'un nombre y par la mthode de Newton-Hron. 
//  cause de la prcision limite, la condition d'arrt peut causer
// de srieux problmes (le tester).  Tout marche, par chance, pour
// a=2, n=2, eps=1e-20, mais tourne  la catastrophe pour a=3.
//
// Afin de rsoudre ce problme il faut remplacer la condition d'arrt.
// Une possibilit sre est de terminer si la prcision souhaite 
// est atteinte ou si la prcision n'augmente plus.
//=======================================================================

#include <iostream>
#include <cmath> 
using namespace std;

typedef float Flo;

Flo puissance( Flo base, int exp )
{
  // Puissance dichotomique, utile surtout pour n grand
  Flo puiss(1);
  for( ; exp > 0 ; puiss*= base, --exp )
    for( ; exp%2 == 0 ; base*= base, exp/= 2 ); 
  return puiss;
}

Flo racine( Flo a, int n, const Flo& eps )
{
  // Attraper des cas triviaux puis prparer les paramtres.
  if ( a <= Flo(0) || n == 0 ) return Flo(0);
  if ( n < 0 ) { a= 1/a; n= -n; }
  if ( n == 1 ) return a;

  // Itrer la mthode de Newton. La condition d'arrt est 
  // trs dlicate  cause des erreurs d'arrondi imprvisibles.
  Flo inf= 1, sup= a, ecart= abs( sup-inf ), ecart_precedent;
  do
    {
      sup= ( (n-1)*sup + inf ) / n;
      inf= a / puissance( sup, n-1 );
      ecart_precedent= ecart;
      ecart= abs( sup-inf );
      cout << inf << " <= racine <= " << sup << " ; cart = " << ecart << endl;
    }
  while( ecart >= eps );
  // while( ecart >= eps && ecart < ecart_precedent );
  return sup;
}

int main()
{
  cout << "\nBonjour et bienvenue au calcul des racines !" << endl
       << "J'essaie de calculer a^(1/n)  une prcision donne." << endl
       << "Entrer a, n et epsilon svp: ";
  Flo a, epsilon;
  int n;
  cin >> a >> n >> epsilon;
  cout << "Je calcule la racine " << n << "ime de " << a 
       << " avec une prcision de " << epsilon << "." << endl;
  
  cout.precision(20);
  cout.setf( ios::scientific | ios::showpoint );
  Flo r= racine( a, n, epsilon );
  cout << "racine   : " << r << endl;
  if ( n == 2 ) cout << "cmath    : " << sqrt( a ) << endl;
  cout << "racine^n : " << puissance(r,n) << endl;
  cout << "Au revoir et  bientt.\n" << endl;
}
