//=======================================================================
// const.cc : la classe RReal et les constantes 
// Copyright (C) 2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Compilation :
// Compiler avec l'option -lgmpxx pour intgrer la bibliothque gmp. 
//
// Description :
// Ce programme illustre comment crire des constantes pour la classe 
// RReal en grande prcision.  
//
// Il y a un pige : une constante littrale comme par exemple  
// 3.1415926535897932384626  est interprte par le compilateur 
// comme tant du type double, donc restreint  53 bits, soit 
// 16 dcimales significatives.  Une fois converti en ce type double,
// les chiffres suivants sont perdus !  
//
// Pour spcifier plus de dcimales dans une constante littrale, 
// il faut l'crire comme une chane de caractres.  Celle-ci est passe 
// au constructeur de la classe  RReal  qui la transforme correctement 
// en un flottant de prcision souhaite.
//=======================================================================

#include <iostream>
#include <iomanip>
using namespace std;

#include "interval.cc"
using namespace Numeric;

// Approximation de pi  ....;....1....;....2....;....3....;....4....;....5
const RReal pi_mino( "3.14159265358979323846264338327950288419716939937510", RNDD, Len(170) );
const RReal pi_majo( "3.14159265358979323846264338327950288419716939937511", RNDU, Len(170) );
const Interval pi( pi_mino, pi_majo );

int main()
{
  cout << "Ce programme sert  illustrer l'usage des constantes prcalcules." << endl
       << "Voici par exemple un encadrement de pi  1e-50 prs :" << endl;

  cout << "\nAffichage de 10 dcimales :" << endl;
  cout.precision(10);
  cout << "pi = " << pi << endl;

  cout << "Affichage de 20 dcimales :" << endl;  
  cout.precision(20);
  cout << "pi = " << pi << endl;

  cout << "Affichage de 30 dcimales :" << endl;
  cout.precision(30);
  cout << "pi = " << pi << endl;

  cout << "Affichage de 40 dcimales :" << endl;
  cout.precision(40);
  cout << "pi = " << pi << endl;

  cout << "Affichage de 50 dcimales :" << endl;
  cout.precision(50);
  cout << "pi = " << pi << endl;

  cout << "Affichage de 60 dcimales (dont 10 non significatives) :" << endl;
  cout.precision(60);
  cout << "pi = " << pi << endl;

  cout << "\nL'usage du type double ne peut donner que 16 dcimales significatives :" << endl;
  RReal a( 3.14159265358979323846264338327950288419716939937510, RNDN, Len(170) );
  cout.precision(50);
  cout << "pi ??? " << a << endl;

  cout << "Un exemple quivalent utilisant explicitement double(3.1414...) :" << endl;
  RReal b( double(3.14159265358979323846264338327950288419716939937510), RNDN, Len(170) );
  cout.precision(50);
  cout << "pi ??? " << b << endl;

  cout << "Un exemple pire encore utilisant explicitement float(3.1414...) :" << endl;
  RReal c( float(3.14159265358979323846264338327950288419716939937510), RNDN, Len(170) );
  cout.precision(50);
  cout << "pi ??? " << c << endl;

  cout << "\nSoulignons  nouveau que l'ordinateur affiche sans aucun remord" << endl 
       << "un nombre quelconque de dcimales, soient-elles significatives ou non.\n" << endl;

  cout << "***************************************************************" << endl
       << "Se mfier de l'affichage inutile de chiffres non significatifs." << endl
       << "***************************************************************\n" << endl;
}

