//=======================================================================
// instable-rreal.cc : itration fiable de l'application  x -> 4x(1-x)
// Copyright (C) 2004-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Compilation :
// Compiler avec l'option -lgmpxx pour intgrer la bibliothque gmp. 
//
// Description :
// Ce petit programme itre la fonction x -> 4x(1-x).  Compte tenu de 
// l'instabilit numrique, on implmente ici un calcul fiable sous forme
// d'encadrements : on calcule l'image f([x,y]) d'un intervalle [x,y] 
// en arrondissant vers le bas pour x et vers le haut pour y.
//
// Commentaire :
// Vous trouvez le mme calcul dans le fichier instable-interval.cc 
// qui profite de l'arithmtique d'intervalles.  
//=======================================================================

#include <iostream>
#include <iomanip>
using namespace std;

#include "rreal.cc"
using namespace Numeric;

// La fonction  f : x -> 4x(1-x)  avec un arrondi fiable
RReal f( const RReal& x, Mode mode= stdMode )
{
  return mul( mul( 4, x, mode), sub( 1, x, mode ), mode );
}

int main()
{
  cout << "\nBonjour et bienvenue ! Ce programme itre la fonction f(x) = 4x(1-x).\n"
       << "Compte tenu de l'instabilit numrique, il semble honnte de calculer\n"
       << "des encadrements fiables grce aux arrondis dirigs.  Ceci ne rsout\n"
       << "pas l'instabilit, mais garantit au moins la correction des calculs.\n" << endl;
  Len decimales= demander_la_precision();
  cout.precision( decimales );
  
  cout << "Donner un intervalle [x,y] pour la valeur initiale :  (par exemple\n"
       << "x=0.1, y=0.1 ne donne pas un point mais un petit intervalle.)" << endl;
  RReal x, y;
  cout << "x = ";
  RReal::mode(RNDD);
  cin >> x;
  cout << "y = ";
  RReal::mode(RNDU);
  cin >> y;
  if ( y < x ) swap( x, y );
  if ( x < 0 || y > 1 ) exit(0);

  cout << "Combien d'itrations dsirez-vous ? ";
  int iterations;
  cin >> iterations;
  
  // On applique t -> 4t(1-t)  l'intervalle [x,y]
  const RReal milieu= RReal(1)/RReal(2);
  for( int k=0; k<=iterations; ++k )
    {
      // L'affichage aussi doit tenir compte des modes d'arrondi
      cout << "k=" << setw(3) << k;
      RReal::mode(RNDD);
      cout << " [ " << x;
      RReal::mode(RNDU);
      cout << " , " << y << " ]" 
	   << "  cart " << (y-x) << endl;
      
      // La fonction f est croissante sur [0,1/2] et dcroissante
      // sur [1/2,0]. Il faut donc distinguer trois cas, suivant
      // la position de l'intervalle [x,y] :
      if ( y <= milieu )         // ici 0 <= x <= y <= 1/2
	{
	  x = f( x, RNDD );
	  y = f( y, RNDU );
	}
      else if ( x >= milieu )    // ici 1/2 <= x <= y <= 1
	{
	  RReal z= x;
	  x= f( y, RNDD );
	  y= f( z, RNDU );
	}
      else                       // ici 0 <= x < 1/2 < y <= 1
	{
	  x= f( x, RNDD );
	  y= f( y, RNDD );
	  if ( y < x ) x= y;
	  y= RReal(1);
	}
    }

  cout << "\nAu revoir et  bientt.\n" << endl;
}
